#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016-07-29
#
# 2016-07-19: monitor Check Point FW Log server status
# 2016-07-24: changed for check state
# 2018-01-05: fix inventory function
# 2018-03-08: changed snmp scan function and inventory function
# 2018-03-15: code cleanup
# 2020-06-08: changed snmp-scan function
# 2021-06-14: rewrite for cmk 2.0, added wato
# 2021-08-09: fixed empty string_table, fixed log server default state
# 2021-08-16: code cleanup
#
# sample snmpwalk
#
# CHECKPOINT-MIB::fwLSConnOverall.0 = INTEGER: 0
# CHECKPOINT-MIB::fwLSConnOverallDesc.0 = STRING: Security Gateway is reporting logs as defined
# CHECKPOINT-MIB::fwLSConnIndex.1.0 = Gauge32: 1
# CHECKPOINT-MIB::fwLSConnIndex.2.0 = Gauge32: 2
# CHECKPOINT-MIB::fwLSConnName.1.0 = STRING: 10.140.2.203
# CHECKPOINT-MIB::fwLSConnName.2.0 = STRING: 10.140.2.103
# CHECKPOINT-MIB::fwLSConnState.1.0 = Gauge32: 0
# CHECKPOINT-MIB::fwLSConnState.2.0 = Gauge32: 2
# CHECKPOINT-MIB::fwLSConnStateDesc.1.0 = STRING: Log-Server Connected
# CHECKPOINT-MIB::fwLSConnStateDesc.2.0 = STRING: Log-Server Disconnected
# CHECKPOINT-MIB::fwLSConnSendRate.1.0 = Gauge32: 0
# CHECKPOINT-MIB::fwLSConnSendRate.2.0 = Gauge32: 0
# CHECKPOINT-MIB::fwLocalLoggingDesc.0 = STRING: Logs are written to log server
# CHECKPOINT-MIB::fwLocalLoggingStat.0 = INTEGER: 0
# CHECKPOINT-MIB::fwLocalLoggingWriteRate.0 = Gauge32: 0
# CHECKPOINT-MIB::fwLoggingHandlingRate.0 = Gauge32: 0
#
# .1.3.6.1.4.1.2620.1.1.30.1.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.1.30.2.0 = STRING: "Security Gateway is reporting logs as defined"
# .1.3.6.1.4.1.2620.1.1.30.3.1.1.1.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.1.30.3.1.1.2.0 = Gauge32: 2
# .1.3.6.1.4.1.2620.1.1.30.3.1.2.1.0 = STRING: "10.140.2.203"
# .1.3.6.1.4.1.2620.1.1.30.3.1.2.2.0 = STRING: "10.140.2.103"
# .1.3.6.1.4.1.2620.1.1.30.3.1.3.1.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.1.30.3.1.3.2.0 = Gauge32: 2
# .1.3.6.1.4.1.2620.1.1.30.3.1.4.1.0 = STRING: "Log-Server Connected"
# .1.3.6.1.4.1.2620.1.1.30.3.1.4.2.0 = STRING: "Log-Server Disconnected"
# .1.3.6.1.4.1.2620.1.1.30.3.1.5.1.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.1.30.3.1.5.2.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.1.30.4.0 = STRING: "Logs are written to log server"
# .1.3.6.1.4.1.2620.1.1.30.5.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.1.30.6.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.1.30.7.0 = Gauge32: 0
#
#
# sample info
# log server running
# [[[u'0', u'Security Gateway is reporting logs as defined', u'Logs are written to log server', u'0']],
#  [[u'1', u'192.168.10.10', u'0', u'Log-Server Connected'],
#   [u'2', u'192.168.10.11', u'2', u'Backup Log-Server Not Active']]]
#
# no log server
# [[], []]
#

from typing import NamedTuple, List, Dict, Optional

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    equals,
    Result,
    State,
    Metric,
    SNMPTree,
    startswith,
    all_of,
    any_of,
)


class CheckPointFwLsOverAll(NamedTuple):
    fwlsconnoverall: int
    fwlsconnoveralldesc: str
    fwlocalloggingdesc: str
    fwlocalloggingstat: int
    fwLocalLoggingWriteRate: Optional[int]
    fwLoggingHandlingRate: Optional[int]


class CheckPointFwLs(NamedTuple):
    fwLSConnIndex: int
    fwLSConnName: str
    fwLSConnState: int
    fwLSConnStateDesc: str
    fwLSConnSendRate: Optional[int]


_fwLSConnState_des = {
    0: 'Ok',
    1: 'Error',
    2: 'Not Active',
}

_fwLocalLoggingStat = {
    0: 'Logging to log servers',
    1: 'Logging local configured',
    2: 'Logging local due to connectivity',
    3: 'Logging local due to high rate',
}


def parse_checkpoint_fw_ls(string_table: List[StringTable]) -> Optional[Dict]:
    over_all, log_servers = string_table
    try:
        fwlsconnoverall, fwlsconnoveralldesc, fwlocalloggingdesc, fwlocalloggingstat, fwlocalloggingwriterate, \
        fwlogginghandlingrate = over_all[0]
    except (IndexError, ValueError):
        return

    parsed = {}
    parsed.update({'over all': CheckPointFwLsOverAll(
        fwlsconnoverall=int(fwlsconnoverall),
        fwlsconnoveralldesc=fwlsconnoveralldesc,
        fwlocalloggingdesc=fwlocalloggingdesc,
        fwlocalloggingstat=int(fwlocalloggingstat),
        fwLocalLoggingWriteRate=int(fwlocalloggingwriterate) if fwlocalloggingwriterate.isdigit() else None,
        fwLoggingHandlingRate=int(fwlogginghandlingrate) if fwlogginghandlingrate.isdigit() else None,
    )})

    for fwLSConnIndex, fwLSConnName, fwLSConnState, fwLSConnStateDesc, fwLSConnSendRate in log_servers:
        parsed.update({fwLSConnName: CheckPointFwLs(
            fwLSConnIndex=int(fwLSConnIndex),
            fwLSConnName=fwLSConnName,
            fwLSConnState=int(fwLSConnState),
            fwLSConnStateDesc=fwLSConnStateDesc,
            fwLSConnSendRate=int(fwLSConnSendRate) if fwLSConnSendRate.isdigit() else None,
        )})

        return parsed


def discovery_checkpoint_fw_ls(section: Dict) -> DiscoveryResult:
    for key in section.keys():
        yield Service(item=key)


def check_checkpoint_fw_ls(item, params, section: Dict) -> CheckResult:
    if item == 'over all':
        over_all = section['over all']

        if over_all.fwLocalLoggingWriteRate is not None:  # R80.10 and up
            yield Metric(name='checkpoint_fw_ls_localloggingwriterate', value=over_all.fwLocalLoggingWriteRate)
        if over_all.fwLoggingHandlingRate is not None:
            yield Metric(name='checkpoint_fw_ls_logginghandlingrate', value=over_all.fwLoggingHandlingRate)

        yield Result(state=State(over_all.fwlsconnoverall), summary=over_all.fwlsconnoveralldesc)
        if over_all.fwlocalloggingdesc != '':
            if over_all.fwlocalloggingstat in [1, 3]:
                yield Result(state=State.WARN, summary=over_all.fwlocalloggingdesc)
            elif over_all.fwlocalloggingstat == 2:
                yield Result(state=State.CRIT, summary=over_all.fwlocalloggingdesc)
            else:
                yield Result(state=State.OK, notice=over_all.fwlocalloggingdesc)
    else:
        try:
            log_server = section[item]
        except IndexError:
            return

        print(params)
        if log_server.fwLSConnSendRate is not None:  # R80.10 and up
            yield Metric(name='checkpoint_fw_ls_lsconnsendrate', value=log_server.fwLSConnSendRate)

        if log_server.fwLSConnState == 1:
            yield Result(state=State.CRIT, summary='State: Connection error')
        else:
            yield Result(state=State.OK, summary=f'State: {log_server.fwLSConnStateDesc}')

        exp_connection_state = params['exp_connection_status']
        mon_connection_state = params['mon_connection_state']

        if log_server.fwLSConnStateDesc.lower() != exp_connection_state.lower():
            yield Result(state=State(mon_connection_state),
                         summary=f'Expected connection state: {exp_connection_state}')


register.snmp_section(
    name='checkpoint_fw_ls',
    parse_function=parse_checkpoint_fw_ls,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.1.30',  # CHECKPOINT-MIB:fwLSConn
            oids=[
                '1',  # fwLSConnOverall
                '2',  # fwLSConnOverallDesc
                '4',  # fwLocalLoggingDesc
                '5',  # fwLocalLoggingStat
                '6',  # fwLocalLoggingWriteRate
                '7',  # fwLoggingHandlingRate
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.1.30.3.1',  # CHECKPOINT-MIB::fwLSConnEntry
            oids=[
                '1',  # fwLSConnIndex
                '2',  # fwLSConnName
                '3',  # fwLSConnState
                '4',  # fwLSConnStateDesc
                '5',  # fwLSConnSendRate
            ]
        )

    ],
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_fw_ls',
    service_name='FW Log server %s',
    discovery_function=discovery_checkpoint_fw_ls,
    check_function=check_checkpoint_fw_ls,
    check_default_parameters={
        'exp_connection_status': 'Log-Server Connected',
        'mon_connection_state': 1,
    },
    check_ruleset_name='checkpoint_fw_ls',
)
